@@ -45,7 +45,7 @@ docker run --rm -i \
   --env ROOT_DIR="${ROOT_DIR}" \
   --env SCRIPT_DIR="${SCRIPT_DIR}" \
   --entrypoint "${SCRIPT_DIR}/build-docker.sh" \
-  "gcr.io/shaderc-build/radial-build:latest"
+  us-east4-docker.pkg.dev/shaderc-build/radial-docker/ubuntu-24.04-amd64/cpp-builder
 
 # This is important. If the permissions are not fixed, kokoro will fail
 # to pull build artifacts, and put the build in tool-failure state, which
